@@ -1,6 +1,6 @@
 /* -*- Mode: c; c-basic-offset: 2 -*-
  *
- * rasqal_expr.c - Rasqal data graph class
+ * rasqal_data_graph.c - Rasqal data graph class
  *
  * Copyright (C) 2010, David Beckett http://www.dajobe.org/
  * 
@@ -52,7 +52,7 @@ rasqal_new_data_graph_common(rasqal_world* world,
 {
   rasqal_data_graph* dg;
 
-  dg = (rasqal_data_graph*)RASQAL_CALLOC(rasqal_data_graph, 1, sizeof(*dg));
+  dg = RASQAL_CALLOC(rasqal_data_graph*, 1, sizeof(*dg));
   if(dg) {
     dg->world = world;
 
@@ -70,18 +70,20 @@ rasqal_new_data_graph_common(rasqal_world* world,
 
     if(format_type) {
       size_t len = strlen(format_type);
-      dg->format_type = RASQAL_MALLOC(string, len + 1);
+      dg->format_type = RASQAL_MALLOC(char*, len + 1);
       if(!dg->format_type)
         goto error;
-      strncpy((char*)dg->format_type, (const char*)format_type, len + 1);
+
+      memcpy(dg->format_type, format_type, len + 1);
     }
 
     if(format_name) {
       size_t len = strlen(format_name);
-      dg->format_name = RASQAL_MALLOC(string, len + 1);
+      dg->format_name = RASQAL_MALLOC(char*, len + 1);
       if(!dg->format_name)
         goto error;
-      strncpy((char*)dg->format_name, (const char*)format_name, len + 1);
+
+      memcpy(dg->format_name, format_name, len + 1);
     }
 
     if(format_uri)
@@ -173,37 +175,6 @@ rasqal_new_data_graph_from_iostream(rasqal_world* world,
 }
 
 
-/**
- * rasqal_new_data_graph:
- * @world: rasqal_world object
- * @uri: source URI
- * @name_uri: name of graph (or NULL)
- * @flags: %RASQAL_DATA_GRAPH_NAMED or %RASQAL_DATA_GRAPH_BACKGROUND
- * 
- * Constructor - create a new #rasqal_data_graph.
- * 
- * The name_uri is only used when the flags are %RASQAL_DATA_GRAPH_NAMED.
- *
- * @Deprecated: replaced by rasqal_new_data_graph_from_uri() with
- * extra format arguments.
- *
- * Return value: a new #rasqal_data_graph or NULL on failure.
- **/
-rasqal_data_graph*
-rasqal_new_data_graph(rasqal_world* world, raptor_uri* uri,
-                      raptor_uri* name_uri, int flags)
-{
-  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(world, rasqal_world, NULL);
-  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(uri, raptor_uri, NULL);
-
-  return rasqal_new_data_graph_common(world,
-                                      uri, 
-                                      /* iostr */ NULL, /* base URI */ NULL,
-                                      name_uri, flags,
-                                      NULL, NULL, NULL);
-}
-
-
 /**
  * rasqal_new_data_graph_from_data_graph:
  * @dg: #rasqal_data_graph object to copy or NULL
@@ -242,9 +213,9 @@ rasqal_free_data_graph(rasqal_data_graph* dg)
   if(dg->name_uri)
     raptor_free_uri(dg->name_uri);
   if(dg->format_type)
-    RASQAL_FREE(cstring, dg->format_type);
+    RASQAL_FREE(char*, dg->format_type);
   if(dg->format_name)
-    RASQAL_FREE(cstring, dg->format_name);
+    RASQAL_FREE(char*, dg->format_name);
   if(dg->format_uri)
     raptor_free_uri(dg->format_uri);
   if(dg->base_uri)
