@@ -44,8 +44,8 @@
 #include <rasqal_internal.h>
 
 
-static int rasqal_query_results_write_sparql_xml(raptor_iostream *iostr, rasqal_query_results* results, raptor_uri *base_uri);
-static rasqal_rowsource* rasqal_query_results_get_rowsource_sparql_xml(rasqal_world *world, rasqal_variables_table* vars_table, raptor_iostream *iostr, raptor_uri *base_uri);
+static int rasqal_query_results_write_sparql_xml(rasqal_query_results_formatter* formatter, raptor_iostream *iostr, rasqal_query_results* results, raptor_uri *base_uri);
+static rasqal_rowsource* rasqal_query_results_get_rowsource_sparql_xml(rasqal_query_results_formatter* formatter, rasqal_world *world, rasqal_variables_table* vars_table, raptor_iostream *iostr, raptor_uri *base_uri, unsigned int flags);
 
 
 #if RASQAL_DEBUG > 1
@@ -79,16 +79,13 @@ static rasqal_rowsource* rasqal_query_results_get_rowsource_sparql_xml(rasqal_wo
  * Return value: non-0 on failure
  **/
 static int
-rasqal_query_results_write_sparql_xml(raptor_iostream *iostr,
+rasqal_query_results_write_sparql_xml(rasqal_query_results_formatter* formatter,
+                                      raptor_iostream *iostr,
                                       rasqal_query_results* results,
                                       raptor_uri *base_uri)
 {
   int rc=1;
   rasqal_world* world = rasqal_query_results_get_world(results);
-#ifndef HAVE_RAPTOR2_API
-  const raptor_uri_handler *uri_handler;
-  void *uri_context;
-#endif
   raptor_xml_writer* xml_writer=NULL;
   raptor_namespace *res_ns=NULL;
   raptor_namespace_stack *nstack=NULL;
@@ -110,57 +107,42 @@ rasqal_query_results_write_sparql_xml(raptor_iostream *iostr,
   }
   
 
-#ifdef HAVE_RAPTOR2_API  
   nstack = raptor_new_namespaces(world->raptor_world_ptr, 1);
-#else
-  raptor_uri_get_handler(&uri_handler, &uri_context);
-  nstack = raptor_new_namespaces(uri_handler, uri_context,
-                                 (raptor_simple_message_handler)rasqal_world_simple_error, world,
-                                 1);
-#endif
   if(!nstack)
     return 1;
 
-#ifdef HAVE_RAPTOR2_API
   xml_writer = raptor_new_xml_writer(world->raptor_world_ptr,
                                      nstack,
                                      iostr);
-#else
-  xml_writer = raptor_new_xml_writer(nstack,
-                                     uri_handler, uri_context,
-                                     iostr,
-                                     (raptor_simple_message_handler)rasqal_world_simple_error, world,
-                                     1);
-#endif
   if(!xml_writer)
     goto tidy;
 
   res_ns=raptor_new_namespace(nstack,
                               NULL,
-                              (const unsigned char*)"http://www.w3.org/2005/sparql-results#",
+                              RASQAL_GOOD_CAST(const unsigned char*, "http://www.w3.org/2005/sparql-results#"),
                               0);
   if(!res_ns)
     goto tidy;
 
   sparql_element=raptor_new_xml_element_from_namespace_local_name(res_ns,
-                                                                  (const unsigned char*)"sparql",
+                                                                  RASQAL_GOOD_CAST(const unsigned char*, "sparql"),
                                                                   NULL, base_uri);
   if(!sparql_element)
     goto tidy;
 
   raptor_xml_writer_start_element(xml_writer, sparql_element);
-  raptor_xml_writer_raw_counted(xml_writer, (const unsigned char*)"\n", 1);
+  raptor_xml_writer_raw_counted(xml_writer, RASQAL_GOOD_CAST(const unsigned char*, "\n"), 1);
 
   /*   <head> */
   element1=raptor_new_xml_element_from_namespace_local_name(res_ns,
-                                                            (const unsigned char*)"head",
+                                                            RASQAL_GOOD_CAST(const unsigned char*, "head"),
                                                             NULL, base_uri);
   if(!element1)
     goto tidy;
 
-  raptor_xml_writer_raw_counted(xml_writer, (const unsigned char*)"  ", 2);
+  raptor_xml_writer_raw_counted(xml_writer, RASQAL_GOOD_CAST(const unsigned char*, "  "), 2);
   raptor_xml_writer_start_element(xml_writer, element1);
-  raptor_xml_writer_raw_counted(xml_writer, (const unsigned char*)"\n", 1);
+  raptor_xml_writer_raw_counted(xml_writer, RASQAL_GOOD_CAST(const unsigned char*, "\n"), 1);
   
   if(rasqal_query_results_is_bindings(results)) {
     for(i=0; 1; i++) {
@@ -171,7 +153,7 @@ rasqal_query_results_write_sparql_xml(raptor_iostream *iostr,
       
       /*     <variable name="x"/> */
       variable_element=raptor_new_xml_element_from_namespace_local_name(res_ns,
-                                                                        (const unsigned char*)"variable",
+                                                                        RASQAL_GOOD_CAST(const unsigned char*,"variable"),
                                                                         NULL, base_uri);
       if(!variable_element)
         goto tidy;
@@ -179,16 +161,10 @@ rasqal_query_results_write_sparql_xml(raptor_iostream *iostr,
       attrs=(raptor_qname **)raptor_alloc_memory(sizeof(raptor_qname*));
       if(!attrs)
         goto tidy;
-#ifdef HAVE_RAPTOR2_API
       attrs[0] = raptor_new_qname_from_namespace_local_name(world->raptor_world_ptr,
                                                             res_ns,
-                                                            (const unsigned char*)"name",
-                                                            (const unsigned char*)name); /* attribute value */
-#else
-      attrs[0] = raptor_new_qname_from_namespace_local_name(res_ns, 
-                                                            (const unsigned char*)"name",
-                                                            (const unsigned char*)name); /* attribute value */
-#endif
+                                                            RASQAL_GOOD_CAST(const unsigned char*,"name"),
+                                                            RASQAL_GOOD_CAST(const unsigned char*, name)); /* attribute value */
       if(!attrs[0]) {
         raptor_free_memory((void*)attrs);
         goto tidy;
@@ -196,9 +172,9 @@ rasqal_query_results_write_sparql_xml(raptor_iostream *iostr,
 
       raptor_xml_element_set_attributes(variable_element, attrs, 1);
       
-      raptor_xml_writer_raw_counted(xml_writer, (const unsigned char*)"    ", 4);
+      raptor_xml_writer_raw_counted(xml_writer, RASQAL_GOOD_CAST(const unsigned char*, "    "), 4);
       raptor_xml_writer_empty_element(xml_writer, variable_element);
-      raptor_xml_writer_raw_counted(xml_writer, (const unsigned char*)"\n", 1);
+      raptor_xml_writer_raw_counted(xml_writer, RASQAL_GOOD_CAST(const unsigned char*, "\n"), 1);
       
       raptor_free_xml_element(variable_element);
       variable_element=NULL;
@@ -209,9 +185,9 @@ rasqal_query_results_write_sparql_xml(raptor_iostream *iostr,
 
     
   /*   </head> */
-  raptor_xml_writer_raw_counted(xml_writer, (const unsigned char*)"  ", 2);
+  raptor_xml_writer_raw_counted(xml_writer, RASQAL_GOOD_CAST(const unsigned char*, "  "), 2);
   raptor_xml_writer_end_element(xml_writer, element1);
-  raptor_xml_writer_raw_counted(xml_writer, (const unsigned char*)"\n", 1);
+  raptor_xml_writer_raw_counted(xml_writer, RASQAL_GOOD_CAST(const unsigned char*, "\n"), 1);
   
   raptor_free_xml_element(element1);
   element1=NULL;
@@ -220,19 +196,19 @@ rasqal_query_results_write_sparql_xml(raptor_iostream *iostr,
   /* Boolean Results */
   if(rasqal_query_results_is_boolean(results)) {
     result_element=raptor_new_xml_element_from_namespace_local_name(res_ns,
-                                                                    (const unsigned char*)"boolean",
+                                                                    RASQAL_GOOD_CAST(const unsigned char*, "boolean"),
                                                                     NULL, base_uri);
     if(!result_element)
       goto tidy;
 
-    raptor_xml_writer_raw_counted(xml_writer, (const unsigned char*)"  ", 2);
+    raptor_xml_writer_raw_counted(xml_writer, RASQAL_GOOD_CAST(const unsigned char*, "  "), 2);
     raptor_xml_writer_start_element(xml_writer, result_element);
     if(rasqal_query_results_get_boolean(results))
       raptor_xml_writer_raw(xml_writer, RASQAL_XSD_BOOLEAN_TRUE);
     else
       raptor_xml_writer_raw(xml_writer, RASQAL_XSD_BOOLEAN_FALSE);
     raptor_xml_writer_end_element(xml_writer, result_element);
-    raptor_xml_writer_raw_counted(xml_writer, (const unsigned char*)"\n", 1);
+    raptor_xml_writer_raw_counted(xml_writer, RASQAL_GOOD_CAST(const unsigned char*, "\n"), 1);
 
     goto results3done;
   }
@@ -242,28 +218,28 @@ rasqal_query_results_write_sparql_xml(raptor_iostream *iostr,
 
   /*   <results> */
   results_element=raptor_new_xml_element_from_namespace_local_name(res_ns,
-                                                                   (const unsigned char*)"results",
+                                                                   RASQAL_GOOD_CAST(const unsigned char*, "results"),
                                                                    NULL, base_uri);
   if(!results_element)
     goto tidy;
 
-  raptor_xml_writer_raw_counted(xml_writer, (const unsigned char*)"  ", 2);
+  raptor_xml_writer_raw_counted(xml_writer, RASQAL_GOOD_CAST(const unsigned char*, "  "), 2);
   raptor_xml_writer_start_element(xml_writer, results_element);
-  raptor_xml_writer_raw_counted(xml_writer, (const unsigned char*)"\n", 1);
+  raptor_xml_writer_raw_counted(xml_writer, RASQAL_GOOD_CAST(const unsigned char*, "\n"), 1);
 
 
   /* declare result element for later multiple use */
   result_element=raptor_new_xml_element_from_namespace_local_name(res_ns,
-                                                                  (const unsigned char*)"result",
+                                                                  RASQAL_GOOD_CAST(const unsigned char*, "result"),
                                                                   NULL, base_uri);
   if(!result_element)
     goto tidy;
 
   while(!rasqal_query_results_finished(results)) {
     /*     <result> */
-    raptor_xml_writer_raw_counted(xml_writer, (const unsigned char*)"    ", 4);
+    raptor_xml_writer_raw_counted(xml_writer, RASQAL_GOOD_CAST(const unsigned char*, "    "), 4);
     raptor_xml_writer_start_element(xml_writer, result_element);
-    raptor_xml_writer_raw_counted(xml_writer, (const unsigned char*)"\n", 1);
+    raptor_xml_writer_raw_counted(xml_writer, RASQAL_GOOD_CAST(const unsigned char*, "\n"), 1);
 
     for(i=0; i<rasqal_query_results_get_bindings_count(results); i++) {
       const unsigned char *name=rasqal_query_results_get_binding_name(results, i);
@@ -271,7 +247,7 @@ rasqal_query_results_write_sparql_xml(raptor_iostream *iostr,
 
       /*       <binding> */
       binding_element=raptor_new_xml_element_from_namespace_local_name(res_ns,
-                                                                       (const unsigned char*)"binding",
+                                                                       RASQAL_GOOD_CAST(const unsigned char*, "binding"),
                                                                        NULL, base_uri);
       if(!binding_element)
         goto tidy;
@@ -279,17 +255,11 @@ rasqal_query_results_write_sparql_xml(raptor_iostream *iostr,
       attrs=(raptor_qname **)raptor_alloc_memory(sizeof(raptor_qname*));
       if(!attrs)
         goto tidy;
-#ifdef HAVE_RAPTOR2_API
       attrs[0] = raptor_new_qname_from_namespace_local_name(world->raptor_world_ptr,
                                                             res_ns,
-                                                            (const unsigned char*)"name",
+                                                            RASQAL_GOOD_CAST(const unsigned char*, "name"),
                                                             name);
 
-#else
-      attrs[0] = raptor_new_qname_from_namespace_local_name(res_ns, 
-                                                            (const unsigned char*)"name",
-                                                            name);
-#endif
       if(!attrs[0]) {
         raptor_free_memory((void*)attrs);
         goto tidy;
@@ -298,12 +268,12 @@ rasqal_query_results_write_sparql_xml(raptor_iostream *iostr,
       raptor_xml_element_set_attributes(binding_element, attrs, 1);
       
 
-      raptor_xml_writer_raw_counted(xml_writer, (const unsigned char*)"      ", 6);
+      raptor_xml_writer_raw_counted(xml_writer, RASQAL_GOOD_CAST(const unsigned char*, "      "), 6);
       raptor_xml_writer_start_element(xml_writer, binding_element);
 
       if(!l) {
         element1=raptor_new_xml_element_from_namespace_local_name(res_ns,
-                                                                  (const unsigned char*)"unbound",
+                                                                  RASQAL_GOOD_CAST(const unsigned char*, "unbound"),
                                                                   NULL, base_uri);
         if(!element1)
           goto tidy;
@@ -312,32 +282,33 @@ rasqal_query_results_write_sparql_xml(raptor_iostream *iostr,
       } else switch(l->type) {
         case RASQAL_LITERAL_URI:
           element1=raptor_new_xml_element_from_namespace_local_name(res_ns,
-                                                                    (const unsigned char*)"uri",
+                                                                    RASQAL_GOOD_CAST(const unsigned char*, "uri"),
                                                                     NULL, base_uri);
           if(!element1)
             goto tidy;
           
           raptor_xml_writer_start_element(xml_writer, element1);
-          raptor_xml_writer_cdata(xml_writer, (const unsigned char*)raptor_uri_as_string(l->value.uri));
+          raptor_xml_writer_cdata(xml_writer, RASQAL_GOOD_CAST(const unsigned char*, raptor_uri_as_string(l->value.uri)));
           raptor_xml_writer_end_element(xml_writer, element1);
 
           break;
 
         case RASQAL_LITERAL_BLANK:
           element1=raptor_new_xml_element_from_namespace_local_name(res_ns,
-                                                                    (const unsigned char*)"bnode",
+                                                                    RASQAL_GOOD_CAST(const unsigned char*, "bnode"),
                                                                     NULL, base_uri);
           if(!element1)
             goto tidy;
           
           raptor_xml_writer_start_element(xml_writer, element1);
-          raptor_xml_writer_cdata(xml_writer, (const unsigned char*)l->string);
+          raptor_xml_writer_cdata(xml_writer, RASQAL_GOOD_CAST(const unsigned char*, l->string));
           raptor_xml_writer_end_element(xml_writer, element1);
           break;
 
         case RASQAL_LITERAL_STRING:
+        case RASQAL_LITERAL_UDT:
           element1=raptor_new_xml_element_from_namespace_local_name(res_ns,
-                                                                    (const unsigned char*)"literal",
+                                                                    RASQAL_GOOD_CAST(const unsigned char*, "literal"),
                                                                     NULL, base_uri);
           if(!element1)
             goto tidy;
@@ -349,23 +320,13 @@ rasqal_query_results_write_sparql_xml(raptor_iostream *iostr,
 
             if(l->language)
               attrs[0]=raptor_new_qname(nstack,
-                                        (const unsigned char*)"xml:lang",
-                                        (const unsigned char*)l->language
-#ifndef HAVE_RAPTOR2_API
-                                        , (raptor_simple_message_handler)rasqal_world_simple_error, world
-#endif
-                                        );
+                                        RASQAL_GOOD_CAST(const unsigned char*, "xml:lang"),
+                                        RASQAL_GOOD_CAST(const unsigned char*, l->language));
             else
-#ifdef HAVE_RAPTOR2_API
               attrs[0] = raptor_new_qname_from_namespace_local_name(world->raptor_world_ptr,
                                                                     res_ns,
-                                                                    (const unsigned char*)"datatype",
-                                                                    (const unsigned char*)raptor_uri_as_string(l->datatype));
-#else
-              attrs[0] = raptor_new_qname_from_namespace_local_name(res_ns, 
-                                                                    (const unsigned char*)"datatype",
-                                                                    (const unsigned char*)raptor_uri_as_string(l->datatype));
-#endif
+                                                                    RASQAL_GOOD_CAST(const unsigned char*, "datatype"),
+                                                                    RASQAL_GOOD_CAST(const unsigned char*, raptor_uri_as_string(l->datatype)));
             if(!attrs[0]) {
               raptor_free_memory((void*)attrs);
               goto tidy;
@@ -379,7 +340,7 @@ rasqal_query_results_write_sparql_xml(raptor_iostream *iostr,
 
 
           raptor_xml_writer_cdata_counted(xml_writer,
-                                          (const unsigned char*)l->string, 
+                                          RASQAL_GOOD_CAST(const unsigned char*, l->string),
                                           l->string_len);
 
           raptor_xml_writer_end_element(xml_writer, element1);
@@ -394,8 +355,8 @@ rasqal_query_results_write_sparql_xml(raptor_iostream *iostr,
         case RASQAL_LITERAL_FLOAT:
         case RASQAL_LITERAL_VARIABLE:
         case RASQAL_LITERAL_DECIMAL:
+        case RASQAL_LITERAL_DATE:
         case RASQAL_LITERAL_DATETIME:
-        case RASQAL_LITERAL_UDT:
         case RASQAL_LITERAL_INTEGER_SUBTYPE:
 
         case RASQAL_LITERAL_UNKNOWN:
@@ -414,29 +375,29 @@ rasqal_query_results_write_sparql_xml(raptor_iostream *iostr,
 
       /*       </binding> */
       raptor_xml_writer_end_element(xml_writer, binding_element);
-      raptor_xml_writer_raw_counted(xml_writer, (const unsigned char*)"\n", 1);
+      raptor_xml_writer_raw_counted(xml_writer, RASQAL_GOOD_CAST(const unsigned char*, "\n"), 1);
       
       raptor_free_xml_element(binding_element);
       binding_element=NULL;
     }
 
-    raptor_xml_writer_raw_counted(xml_writer, (const unsigned char*)"    ", 4);
+    raptor_xml_writer_raw_counted(xml_writer, RASQAL_GOOD_CAST(const unsigned char*, "    "), 4);
     raptor_xml_writer_end_element(xml_writer, result_element);
-    raptor_xml_writer_raw_counted(xml_writer, (const unsigned char*)"\n", 1);
+    raptor_xml_writer_raw_counted(xml_writer, RASQAL_GOOD_CAST(const unsigned char*, "\n"), 1);
     
     rasqal_query_results_next(results);
   }
 
-  raptor_xml_writer_raw_counted(xml_writer, (const unsigned char*)"  ", 2);
+  raptor_xml_writer_raw_counted(xml_writer, RASQAL_GOOD_CAST(const unsigned char*, "  "), 2);
   raptor_xml_writer_end_element(xml_writer, results_element);
-  raptor_xml_writer_raw_counted(xml_writer, (const unsigned char*)"\n", 1);
+  raptor_xml_writer_raw_counted(xml_writer, RASQAL_GOOD_CAST(const unsigned char*, "\n"), 1);
 
   results3done:
 
   rc=0;
 
   raptor_xml_writer_end_element(xml_writer, sparql_element);
-  raptor_xml_writer_raw_counted(xml_writer, (const unsigned char*)"\n", 1);
+  raptor_xml_writer_raw_counted(xml_writer, RASQAL_GOOD_CAST(const unsigned char*, "\n"), 1);
 
   tidy:
   if(element1)
@@ -515,9 +476,6 @@ typedef struct
   raptor_sax2* sax2;
   raptor_locator locator;
   int depth; /* element depth */
-#ifndef HAVE_RAPTOR2_API
-  raptor_error_handlers error_handlers; /* SAX2 error handler */
-#endif
 
   /* SPARQL XML Results parsing */
   rasqal_sparql_xml_read_state state; /* state */
@@ -539,6 +497,8 @@ typedef struct
   /* Variables table allocated for variables in the result set */
   rasqal_variables_table* vars_table;
   int variables_count;
+
+  unsigned int flags;
 } rasqal_rowsource_sparql_xml_context;
   
 
@@ -567,7 +527,7 @@ rasqal_sparql_xml_sax2_start_element_handler(void *user_data,
   name=raptor_xml_element_get_name(xml_element);
 
   for(i=STATE_first; i <= STATE_last; i++) {
-    if(!strcmp((const char*)raptor_qname_get_local_name(name),
+    if(!strcmp(RASQAL_GOOD_CAST(const char*, raptor_qname_get_local_name(name)),
                sparql_xml_element_names[i])) {
       state=(rasqal_sparql_xml_read_state)i;
       con->state=state;
@@ -589,31 +549,31 @@ rasqal_sparql_xml_sax2_start_element_handler(void *user_data,
   
   attr_count=raptor_xml_element_get_attributes_count(xml_element);
   con->name=NULL;
+  con->value = NULL;
+  con->value_len = 0;
   con->datatype=NULL;
   con->language=NULL;
   
   if(attr_count > 0) {
     raptor_qname** attrs=raptor_xml_element_get_attributes(xml_element);
-    for(i=0; i < attr_count; i++) {
+    for(i = 0; i < attr_count; i++) {
+      const char* local_name = RASQAL_GOOD_CAST(const char*, raptor_qname_get_local_name(attrs[i]));
 #ifdef TRACE_XML
       if(con->trace) {
         pad(stderr, con->depth+1);
         fprintf(stderr, "Attribute %s='%s'\n",
-                raptor_qname_get_local_name(attrs[i]),
-                raptor_qname_get_value(attrs[i]));
+                local_name, raptor_qname_get_value(attrs[i]));
       }
 #endif
-      if(!strcmp((const char*)raptor_qname_get_local_name(attrs[i]),
-                 "name"))
-        con->name=(const char*)raptor_qname_get_counted_value(attrs[i],
-                                                             &con->name_length);
-      else if(!strcmp((const char*)raptor_qname_get_local_name(attrs[i]),
-                      "datatype"))
-        con->datatype=(const char*)raptor_qname_get_value(attrs[i]);
+
+      if(!strcmp(local_name, "name"))
+        con->name = RASQAL_GOOD_CAST(const char*, raptor_qname_get_counted_value(attrs[i], &con->name_length));
+      else if(!strcmp(local_name, "datatype"))
+        con->datatype = RASQAL_GOOD_CAST(const char*, raptor_qname_get_value(attrs[i]));
     }
   }
   if(raptor_xml_element_get_language(xml_element)) {
-    con->language=(const char*)raptor_xml_element_get_language(xml_element);
+    con->language = RASQAL_GOOD_CAST(const char*, raptor_xml_element_get_language(xml_element));
 #ifdef TRACE_XML
     if(con->trace) {
       pad(stderr, con->depth+1);
@@ -624,31 +584,34 @@ rasqal_sparql_xml_sax2_start_element_handler(void *user_data,
 
   switch(state) {
     case STATE_variable:
-      if(1) {
+      if(con->name) {
         unsigned char* var_name;
         rasqal_variable *v;
         
-        var_name = (unsigned char*)RASQAL_MALLOC(cstring, con->name_length+1);
+        var_name = RASQAL_MALLOC(unsigned char*, con->name_length + 1);
         memcpy(var_name, con->name, con->name_length + 1);
 
         v = rasqal_variables_table_add(con->vars_table,
                                        RASQAL_VARIABLE_TYPE_NORMAL,
                                        var_name, NULL);
-        if(v)
+        if(v) {
           rasqal_rowsource_add_variable(con->rowsource, v);
+          /* above function takes a reference to v */
+          rasqal_free_variable(v);
+        }
       }
       break;
       
     case STATE_result:
       if(1) {
-        con->row=rasqal_new_row(con->rowsource);
+        con->row = rasqal_new_row(con->rowsource);
         RASQAL_DEBUG2("Made new row %d\n", con->offset);
         con->offset++;
       }
       break;
       
     case STATE_binding:
-      con->result_offset = rasqal_rowsource_get_variable_offset_by_name(con->rowsource, (const unsigned char*)con->name);
+      con->result_offset = rasqal_rowsource_get_variable_offset_by_name(con->rowsource, RASQAL_GOOD_CAST(const unsigned char*, con->name));
       break;
       
     case STATE_sparql:
@@ -686,8 +649,8 @@ rasqal_sparql_xml_sax2_characters_handler(void *user_data,
   if(con->state == STATE_literal ||
      con->state == STATE_uri ||
      con->state == STATE_bnode) {
-    con->value_len=len;
-    con->value=(char*)RASQAL_MALLOC(cstring, len+1);
+    con->value_len = len;
+    con->value = RASQAL_MALLOC(char*, len + 1);
     memcpy(con->value, s, len);
     con->value[len]='\0';
   }
@@ -708,7 +671,7 @@ rasqal_sparql_xml_sax2_end_element_handler(void *user_data,
   name=raptor_xml_element_get_name(xml_element);
 
   for(i=STATE_first; i <= STATE_last; i++) {
-    if(!strcmp((const char*)raptor_qname_get_local_name(name),
+    if(!strcmp(RASQAL_GOOD_CAST(const char*, raptor_qname_get_local_name(name)),
                sparql_xml_element_names[i])) {
       state=(rasqal_sparql_xml_read_state)i;
       con->state=state;
@@ -743,13 +706,17 @@ rasqal_sparql_xml_sax2_end_element_handler(void *user_data,
         raptor_uri* datatype_uri=NULL;
         char* language_str=NULL;
 
-        lvalue=(unsigned char*)RASQAL_MALLOC(cstring, con->value_len+1);
-        memcpy(lvalue, con->value, con->value_len + 1);
+        lvalue = RASQAL_MALLOC(unsigned char*, con->value_len + 1);
+        if(!con->value_len)
+          *lvalue = '\0';
+        else
+          memcpy(lvalue, con->value, con->value_len + 1);
         if(con->datatype)
-          datatype_uri = raptor_new_uri(con->world->raptor_world_ptr, (const unsigned char*)con->datatype);
+          datatype_uri = raptor_new_uri(con->world->raptor_world_ptr, RASQAL_GOOD_CAST(const unsigned char*, con->datatype));
         if(con->language) {
-          language_str=(char*)RASQAL_MALLOC(cstring, strlen(con->language)+1);
-          strcpy(language_str, con->language);
+          size_t language_len = strlen(con->language);
+          language_str = RASQAL_MALLOC(char*, language_len + 1);
+          memcpy(language_str, con->language, language_len + 1);
         }
         l = rasqal_new_string_literal_node(con->world, lvalue, language_str,
                                            datatype_uri);
@@ -764,7 +731,7 @@ rasqal_sparql_xml_sax2_end_element_handler(void *user_data,
       if(1) {
         rasqal_literal* l;
         unsigned char* lvalue;
-        lvalue=(unsigned char*)RASQAL_MALLOC(cstring, con->value_len+1);
+        lvalue = RASQAL_MALLOC(unsigned char*, con->value_len + 1);
         memcpy(lvalue, con->value, con->value_len + 1);
         l = rasqal_new_simple_literal(con->world, RASQAL_LITERAL_BLANK, lvalue);
         rasqal_row_set_value_at(con->row, con->result_offset, l);
@@ -778,7 +745,7 @@ rasqal_sparql_xml_sax2_end_element_handler(void *user_data,
       if(1) {
         raptor_uri* uri;
         rasqal_literal* l;
-        uri = raptor_new_uri(con->world->raptor_world_ptr, (const unsigned char*)con->value);
+        uri = raptor_new_uri(con->world->raptor_world_ptr, RASQAL_GOOD_CAST(const unsigned char*, con->value));
         l = rasqal_new_uri_literal(con->world, uri);
         rasqal_row_set_value_at(con->row, con->result_offset, l);
         rasqal_free_literal(l);
@@ -805,7 +772,7 @@ rasqal_sparql_xml_sax2_end_element_handler(void *user_data,
   }
 
   if(con->value) {
-    RASQAL_FREE(cstring, con->value);
+    RASQAL_FREE(char*, con->value);
     con->value=NULL;
   }
 }
@@ -852,6 +819,11 @@ rasqal_rowsource_sparql_xml_finish(rasqal_rowsource* rowsource, void *user_data)
   if(con->vars_table)
     rasqal_free_variables_table(con->vars_table);
 
+  if(con->flags) {
+    if(con->iostr)
+      raptor_free_iostream(con->iostr);
+  }
+
   RASQAL_FREE(rasqal_rowsource_sparql_xml_context, con);
 
   return 0;
@@ -868,13 +840,12 @@ rasqal_rowsource_sparql_xml_process(rasqal_rowsource_sparql_xml_context* con)
   while(!raptor_iostream_read_eof(con->iostr)) {
     size_t read_len;
     
-    read_len = raptor_iostream_read_bytes((char*)con->buffer, 1,
+    read_len = raptor_iostream_read_bytes(RASQAL_GOOD_CAST(char*, con->buffer), 1,
                                           FILE_READ_BUF_SIZE,
                                           con->iostr);
     if(read_len > 0) {
-      RASQAL_DEBUG2("processing %d bytes\n", (int)read_len);
+      RASQAL_DEBUG2("processing %d bytes\n", RASQAL_GOOD_CAST(int, read_len));
       raptor_sax2_parse_chunk(con->sax2, con->buffer, read_len, 0);
-      con->locator.byte += read_len;
     }
     
     if(read_len < FILE_READ_BUF_SIZE) {
@@ -935,7 +906,7 @@ static const rasqal_rowsource_handler rasqal_rowsource_sparql_xml_handler={
   /* .read_row = */ rasqal_rowsource_sparql_xml_read_row,
   /* .read_all_rows = */ NULL,
   /* .reset = */ NULL,
-  /* .set_preserve = */ NULL,
+  /* .set_requirements = */ NULL,
   /* .get_inner_rowsource = */ NULL,
   /* .set_origin = */ NULL,
 };
@@ -954,14 +925,16 @@ static const rasqal_rowsource_handler rasqal_rowsource_sparql_xml_handler={
  * Return value: a new rasqal_rowsource or NULL on failure
  **/
 static rasqal_rowsource*
-rasqal_query_results_get_rowsource_sparql_xml(rasqal_world *world,
+rasqal_query_results_get_rowsource_sparql_xml(rasqal_query_results_formatter* formatter,
+                                              rasqal_world *world,
                                               rasqal_variables_table* vars_table,
                                               raptor_iostream *iostr,
-                                              raptor_uri *base_uri)
+                                              raptor_uri *base_uri,
+                                              unsigned int flags)
 {
   rasqal_rowsource_sparql_xml_context* con;
   
-  con=(rasqal_rowsource_sparql_xml_context*)RASQAL_CALLOC(rasqal_rowsource_sparql_xml_context, 1, sizeof(rasqal_rowsource_sparql_xml_context));
+  con = RASQAL_CALLOC(rasqal_rowsource_sparql_xml_context*, 1, sizeof(*con));
   if(!con)
     return NULL;
 
@@ -971,16 +944,11 @@ rasqal_query_results_get_rowsource_sparql_xml(rasqal_world *world,
 
   con->locator.uri=base_uri;
 
-#ifdef HAVE_RAPTOR2_API
   con->sax2 = raptor_new_sax2(world->raptor_world_ptr, &con->locator, con);
-#else
-  con->error_handlers.locator = &con->locator;
-  raptor_error_handlers_init(&con->error_handlers);
-
-  con->sax2 = raptor_new_sax2(con, &con->error_handlers);
-#endif
   if(!con->sax2)
     return NULL;
+
+  con->flags = flags;
   
   raptor_sax2_set_start_element_handler(con->sax2,
                                         rasqal_sparql_xml_sax2_start_element_handler);
@@ -990,11 +958,7 @@ rasqal_query_results_get_rowsource_sparql_xml(rasqal_world *world,
   raptor_sax2_set_end_element_handler(con->sax2,
                                       rasqal_sparql_xml_sax2_end_element_handler);
 
-#ifdef HAVE_RAPTOR2_API
   con->results_sequence = raptor_new_sequence((raptor_data_free_handler)rasqal_free_row, (raptor_data_print_handler)rasqal_row_print);
-#else
-  con->results_sequence = raptor_new_sequence((raptor_sequence_free_handler*)rasqal_free_row, (raptor_sequence_print_handler*)rasqal_row_print);
-#endif
 
   con->vars_table = rasqal_new_variables_table_from_variables_table(vars_table);
   
@@ -1007,35 +971,61 @@ rasqal_query_results_get_rowsource_sparql_xml(rasqal_world *world,
 
 
 
-int
-rasqal_init_result_format_sparql_xml(rasqal_world* world)
+static int
+rasqal_query_results_xml_recognise_syntax(rasqal_query_results_format_factory* factory, 
+                                          const unsigned char *buffer, 
+                                          size_t len,
+                                          const unsigned char *identifier,
+                                          const unsigned char *suffix,
+const char *mime_type)
 {
-  rasqal_query_results_formatter_func writer_fn=NULL;
-  rasqal_query_results_formatter_func reader_fn=NULL;
-  rasqal_query_results_get_rowsource_func get_rowsource_fn=NULL;
-  int rc=0;
 
-  /*
-   * SPARQL XML Results 2008-01-15
-   * http://www.w3.org/TR/2008/REC-rdf-sparql-XMLres-20080115/
-   */
-  writer_fn=&rasqal_query_results_write_sparql_xml;
-  reader_fn=NULL,
-  get_rowsource_fn=&rasqal_query_results_get_rowsource_sparql_xml;
-  rc+= rasqal_query_results_format_register_factory(world,
-                                                    "xml",
-                                                    "SPARQL XML Query Results",
-                                                    (unsigned char*)"http://www.w3.org/2005/sparql-results#",
-                                                    writer_fn, reader_fn, get_rowsource_fn,
-                                                    "application/sparql-results+xml")
-                                                    != 0;
-  rc+= rasqal_query_results_format_register_factory(world,
-                                                    NULL,
-                                                    NULL,
-                                                    (unsigned char*)"http://www.w3.org/TR/2008/REC-rdf-sparql-XMLres-20080115/",
-                                                    writer_fn, reader_fn, get_rowsource_fn,
-                                                    "application/sparql-results+xml")
-                                                    != 0;
+  if(suffix && !strcmp(RASQAL_GOOD_CAST(const char*, suffix), "srx"))
+    return 8;
+  
+  return 0;
+}
+
+
+static const char* const sparql_xml_names[] = { "xml", NULL};
+
+static const char* const sparql_xml_uri_strings[] = {
+  "http://www.w3.org/ns/formats/SPARQL_Results_XML",
+  "http://www.w3.org/TR/rdf-sparql-XMLres/",
+  "http://www.w3.org/TR/2008/REC-rdf-sparql-XMLres-20080115/",
+  "http://www.w3.org/2005/sparql-results#",
+  NULL
+};
+
+static const raptor_type_q sparql_xml_types[] = {
+  { "application/sparql-results+xml", 30, 10}, 
+  { NULL, 0, 0}
+};
+
+static int
+rasqal_query_results_sparql_xml_register_factory(rasqal_query_results_format_factory *factory) 
+{
+  int rc = 0;
+
+  factory->desc.names = sparql_xml_names;
+  factory->desc.mime_types = sparql_xml_types;
+
+  factory->desc.label = "SPARQL XML Query Results";
+  factory->desc.uri_strings = sparql_xml_uri_strings;
+
+  factory->desc.flags = 0;
+  
+  factory->write         = rasqal_query_results_write_sparql_xml;
+  factory->get_rowsource = rasqal_query_results_get_rowsource_sparql_xml;
+  factory->recognise_syntax = rasqal_query_results_xml_recognise_syntax;
+
   return rc;
 }
 
+
+int
+rasqal_init_result_format_sparql_xml(rasqal_world* world)
+{
+  return !rasqal_world_register_query_results_format_factory(world,
+                                                             &rasqal_query_results_sparql_xml_register_factory);
+}
