@@ -71,7 +71,7 @@ typedef struct {
   /* genid base for mapping user bnodes */
   unsigned char* mapped_id_base;
   /* length of above string */
-  int mapped_id_base_len;
+  size_t mapped_id_base_len;
 } rasqal_raptor_triples_source_user_data;
 
 
@@ -81,33 +81,7 @@ static int rasqal_raptor_triple_present(rasqal_triples_source *rts, void *user_d
 static void rasqal_raptor_free_triples_source(void *user_data);
 
 
-#ifndef HAVE_RAPTOR2_API
-static raptor_uri* 
-ordinal_as_uri(rasqal_world* world, int ordinal) 
-{
-  int t = ordinal;
-  size_t len; 
-  unsigned char *buffer;
-  raptor_uri* uri;
-  
-  len = raptor_rdf_namespace_uri_len + 1 + 1; /* 1 for min-length, 1 for '_' */
-  while(t /= 10)
-    len++;
-  buffer = (unsigned char*)RASQAL_MALLOC(cstring, len + 1);
-  if(!buffer)
-    return NULL;
-  
-  sprintf((char*)buffer, "%s_%d", raptor_rdf_namespace_uri, ordinal);
-  uri = raptor_new_uri(world->raptor_world_ptr, buffer);
-  RASQAL_FREE(cstring, buffer);
-
-  return uri;
-}
-#endif
-
-
-#ifdef HAVE_RAPTOR2_API
-static rasqal_triple*
+rasqal_triple*
 raptor_statement_as_rasqal_triple(rasqal_world* world,
                                   const raptor_statement *statement)
 {
@@ -118,8 +92,8 @@ raptor_statement_as_rasqal_triple(rasqal_world* world,
 
   /* subject */
   if(statement->subject->type == RAPTOR_TERM_TYPE_BLANK) {
-    len = strlen((char*)statement->subject->value.blank.string);
-    new_str = (unsigned char*)RASQAL_MALLOC(cstring, len + 1);
+    len = statement->subject->value.blank.string_len;
+    new_str = RASQAL_MALLOC(unsigned char*, len + 1);
     memcpy(new_str, statement->subject->value.blank.string, len + 1);
     s = rasqal_new_simple_literal(world, RASQAL_LITERAL_BLANK, new_str);
   } else {
@@ -135,13 +109,13 @@ raptor_statement_as_rasqal_triple(rasqal_world* world,
   if(statement->object->type == RAPTOR_TERM_TYPE_LITERAL) {
     char *language = NULL;
 
-    len = strlen((char*)statement->object->value.literal.string);
-    new_str = (unsigned char*)RASQAL_MALLOC(cstring, len + 1);
+    len = statement->object->value.literal.string_len;
+    new_str = RASQAL_MALLOC(unsigned char*, len + 1);
     memcpy(new_str, statement->object->value.literal.string, len + 1);
 
     if(statement->object->value.literal.language) {
-      len = strlen((const char*)statement->object->value.literal.language);
-      language = (char*)RASQAL_MALLOC(cstring, len + 1);
+      len = statement->object->value.literal.language_len;
+      language = RASQAL_MALLOC(char*, len + 1);
       memcpy(language, statement->object->value.literal.language, len + 1);
     }
 
@@ -152,8 +126,8 @@ raptor_statement_as_rasqal_triple(rasqal_world* world,
 
     o = rasqal_new_string_literal(world, new_str, language, uri, NULL);
   } else if(statement->object->type == RAPTOR_TERM_TYPE_BLANK) {
-    len = strlen((char*)statement->object->value.blank.string);
-    new_str = (unsigned char*)RASQAL_MALLOC(cstring, len + 1);
+    len = statement->object->value.blank.string_len;
+    new_str = RASQAL_MALLOC(unsigned char*, len + 1);
     memcpy(new_str, statement->object->value.blank.string, len + 1);
     o = rasqal_new_simple_literal(world, RASQAL_LITERAL_BLANK, new_str);
   } else {
@@ -163,108 +137,10 @@ raptor_statement_as_rasqal_triple(rasqal_world* world,
 
   return rasqal_new_triple(s, p, o);
 }
-#else /* ifdef HAVE_RAPTOR_API */
-static rasqal_triple*
-raptor_statement_as_rasqal_triple(rasqal_world* world,
-                                  const raptor_statement *statement)
-{
-  rasqal_literal *s, *p, *o;
-
-  if(statement->subject_type == RAPTOR_IDENTIFIER_TYPE_ANONYMOUS) {
-    unsigned char *new_blank;
-    size_t blank_len;
-
-    blank_len = strlen((char*)statement->subject);
-    new_blank = (unsigned char*)RASQAL_MALLOC(cstring, blank_len + 1);
-    memcpy(new_blank, statement->subject, blank_len + 1);
-    s = rasqal_new_simple_literal(world, RASQAL_LITERAL_BLANK, new_blank);
-  } else if(statement->subject_type == RAPTOR_IDENTIFIER_TYPE_ORDINAL) {
-    raptor_uri* uri;
-
-    uri = ordinal_as_uri(world, *((int*)statement->subject));
-    if(!uri)
-      return NULL;
-    s = rasqal_new_uri_literal(world, uri);
-  } else {
-    raptor_uri *uri;
-
-    uri = raptor_uri_copy((raptor_uri*)statement->subject);
-    s = rasqal_new_uri_literal(world, uri);
-  }
-
-  if(statement->predicate_type == RAPTOR_IDENTIFIER_TYPE_ORDINAL) {
-    raptor_uri* uri = ordinal_as_uri(world, *((int*)statement->predicate));
-
-    if(!uri)
-      return NULL;
-    p = rasqal_new_uri_literal(world, uri);
-  } else {
-    raptor_uri *uri;
-
-    uri = raptor_uri_copy((raptor_uri*)statement->predicate);
-    p = rasqal_new_uri_literal(world, uri);
-  }
-  
-  if(statement->object_type == RAPTOR_IDENTIFIER_TYPE_LITERAL || 
-     statement->object_type == RAPTOR_IDENTIFIER_TYPE_XML_LITERAL) {
-    unsigned char *string;
-    char *language = NULL;
-    raptor_uri *uri = NULL;
-    size_t literal_len;
-
-    literal_len = strlen((char*)statement->object);
-    string = (unsigned char*)RASQAL_MALLOC(cstring, literal_len + 1);
-    memcpy(string, statement->object, literal_len + 1);
-
-    if(statement->object_literal_language) {
-      size_t language_len;
-      language_len = strlen((const char*)statement->object_literal_language);
-      language = (char*)RASQAL_MALLOC(cstring, language_len + 1);
-      memcpy(language, statement->object_literal_language, language_len + 1);
-    }
-
-    if(statement->object_type == RAPTOR_IDENTIFIER_TYPE_XML_LITERAL) {
-      const unsigned char* dt_uri_string;
-
-      dt_uri_string = (const unsigned char*)raptor_xml_literal_datatype_uri_string;
-      uri = raptor_new_uri(world->raptor_world_ptr, dt_uri_string);
-    } else if(statement->object_literal_datatype) {
-      raptor_uri *dt_uri = (raptor_uri*)statement->object_literal_datatype;
-
-      uri = raptor_uri_copy(dt_uri);
-    }
-    o = rasqal_new_string_literal(world, string, language, uri, NULL);
-  } else if(statement->object_type == RAPTOR_IDENTIFIER_TYPE_ANONYMOUS) {
-    char *blank = (char*)statement->object;
-    unsigned char *new_blank;
-    size_t blank_len;
-
-    blank_len = strlen(blank);
-    new_blank = (unsigned char*)RASQAL_MALLOC(cstring, blank_len + 1);
-    memcpy(new_blank, blank, blank_len + 1);
-    o = rasqal_new_simple_literal(world, RASQAL_LITERAL_BLANK, new_blank);
-  } else if(statement->object_type == RAPTOR_IDENTIFIER_TYPE_ORDINAL) {
-    raptor_uri* uri = ordinal_as_uri(world, *((int*)statement->object));
-
-    if(!uri)
-      return NULL;
-    o = rasqal_new_uri_literal(world, uri);
-  } else {
-    raptor_uri *uri;
-    uri = raptor_uri_copy((raptor_uri*)statement->object);
-    o = rasqal_new_uri_literal(world, uri);
-  }
-
-  return rasqal_new_triple(s, p, o);
-}
-#endif /* ifdef HAVE_RAPTOR2_API ... else */
 
 
 static void
 rasqal_raptor_statement_handler(void *user_data,
-#ifndef HAVE_RAPTOR2_API
-                                const
-#endif
                                 raptor_statement *statement)
 {
   rasqal_raptor_triples_source_user_data* rtsc;
@@ -272,8 +148,7 @@ rasqal_raptor_statement_handler(void *user_data,
   
   rtsc = (rasqal_raptor_triples_source_user_data*)user_data;
 
-  triple = (rasqal_raptor_triple*)RASQAL_MALLOC(rasqal_raptor_triple,
-                                                sizeof(rasqal_raptor_triple));
+  triple = RASQAL_MALLOC(rasqal_raptor_triple*, sizeof(rasqal_raptor_triple));
   triple->next = NULL;
   triple->triple = raptor_statement_as_rasqal_triple(rtsc->query->world,
                                                      statement);
@@ -293,48 +168,8 @@ rasqal_raptor_statement_handler(void *user_data,
 }
 
 
-#ifndef HAVE_RAPTOR2_API
-static void
-rasqal_raptor_error_handler(void *user_data, 
-                            raptor_locator* locator, const char *message)
-{
-  rasqal_query* query = (rasqal_query*)user_data;
-  int locator_len;
-
-  query->failed = 1;
-
-  if(locator &&
-#ifdef HAVE_RAPTOR2_API
-     (locator_len = raptor_locator_format(NULL, 0, locator)) > 0
-#else
-     (locator_len = raptor_format_locator(NULL, 0, locator)) > 0
-#endif
-    ) {
-    char *buffer = (char*)RASQAL_MALLOC(cstring, locator_len + 1);
-#ifdef HAVE_RAPTOR2_API
-    raptor_locator_format(buffer, locator_len, locator);
-#else
-    raptor_format_locator(buffer, locator_len, locator);
-#endif
-
-    rasqal_log_error_simple(query->world, RAPTOR_LOG_LEVEL_ERROR,
-                            &query->locator,
-                            "Failed to parse %s - %s", buffer, message);
-    RASQAL_FREE(cstring, buffer);
-  } else
-    rasqal_log_error_simple(query->world, RAPTOR_LOG_LEVEL_ERROR,
-                            &query->locator,
-                            "Failed to parse - %s", message);
-}
-#endif
-
-
 static unsigned char*
 rasqal_raptor_generate_id_handler(void *user_data,
-#ifdef HAVE_RAPTOR2_API
-#else
-                                  raptor_genid_type type,
-#endif
                                   unsigned char *user_bnodeid) 
 {
   rasqal_raptor_triples_source_user_data* rtsc;
@@ -343,11 +178,10 @@ rasqal_raptor_generate_id_handler(void *user_data,
 
   if(user_bnodeid) {
     unsigned char *mapped_id;
-    size_t user_bnodeid_len = strlen((const char*)user_bnodeid);
+    size_t user_bnodeid_len = strlen(RASQAL_GOOD_CAST(const char*, user_bnodeid));
     
-    mapped_id = (unsigned char*)RASQAL_MALLOC(cstring, 
-                                              rtsc->mapped_id_base_len + 1 + 
-                                              user_bnodeid_len + 1);
+    mapped_id = RASQAL_MALLOC(unsigned char*, 
+                              rtsc->mapped_id_base_len + 1 + user_bnodeid_len + 1);
     memcpy(mapped_id, rtsc->mapped_id_base, rtsc->mapped_id_base_len);
     mapped_id[rtsc->mapped_id_base_len] = '_';
     memcpy(mapped_id + rtsc->mapped_id_base_len + 1,
@@ -357,7 +191,7 @@ rasqal_raptor_generate_id_handler(void *user_data,
     return mapped_id;
   }
   
-  return rasqal_query_get_genid(rtsc->query, (const unsigned char*)"genid", -1);
+  return rasqal_query_get_genid(rtsc->query, RASQAL_GOOD_CAST(const unsigned char*, "genid"), -1);
 }
 
 
@@ -377,42 +211,6 @@ rasqal_raptor_support_feature(void *user_data,
 }
 
 
-#ifdef HAVE_RAPTOR2_API
-#else
-
-#define RASQAL_RAPTOR_READ_BUFFER_SIZE 4096
-
-/* Based on raptor_parser_parse_iostream() from Raptor 1.9.0
- * backported to the Raptor V1 API
- */
-static int
-rasqal_raptor_parse_iostream(raptor_parser* rdf_parser, raptor_iostream *iostr,
-                             raptor_uri *base_uri)
-{
-  /* Read buffer */
-  unsigned char buffer[RASQAL_RAPTOR_READ_BUFFER_SIZE + 1];
-  int rc = 0;
-
-  rc = raptor_start_parse(rdf_parser, base_uri);
-  if(rc)
-    return rc;
-  
-  while(!raptor_iostream_read_eof(iostr)) {
-    int len = raptor_iostream_read_bytes(buffer, 1,
-                                         RASQAL_RAPTOR_READ_BUFFER_SIZE,
-                                         iostr);
-    int is_end = (len < RASQAL_RAPTOR_READ_BUFFER_SIZE);
-
-    rc = raptor_parse_chunk(rdf_parser, buffer, len, is_end);
-    if(rc || is_end)
-      break;
-  }
-  
-  return rc;
-}
-#endif
-
-
 static int
 rasqal_raptor_init_triples_source(rasqal_query* rdf_query,
                                   void *factory_user_data,
@@ -426,12 +224,6 @@ rasqal_raptor_init_triples_source(rasqal_query* rdf_query,
 
   rtsc = (rasqal_raptor_triples_source_user_data*)user_data;
 
-  if(!rdf_query->data_graphs) {
-    if(handler)
-      handler(rdf_query, /* locator */ NULL, "Query has no data graphs.");
-    return -1;  /* no data */
-  }
-
   /* Max API version this triples source generates */
   rts->version = 2;
   
@@ -440,17 +232,16 @@ rasqal_raptor_init_triples_source(rasqal_query* rdf_query,
   rts->free_triples_source = rasqal_raptor_free_triples_source;
   rts->support_feature = rasqal_raptor_support_feature;
 
-  rtsc->sources_count = raptor_sequence_size(rdf_query->data_graphs);
-  /* no default triple source possible */
-  if(!rtsc->sources_count) {
-    if(handler)
-      handler(rdf_query, /* locator */ NULL, "Query has no data graphs.");
-    return -1;  /* no data */
-  }
-
-  rtsc->source_literals = (rasqal_literal**)RASQAL_CALLOC(rasqal_literal_ptr,
-                                                          rtsc->sources_count,
-                                                          sizeof(rasqal_literal*));
+  if(rdf_query->data_graphs)
+    rtsc->sources_count = raptor_sequence_size(rdf_query->data_graphs);
+  else
+    /* No data graph - assume there is just a background graph */
+    rtsc->sources_count = 0;
+  
+  if(rtsc->sources_count)
+    rtsc->source_literals = RASQAL_CALLOC(rasqal_literal**, rtsc->sources_count, sizeof(rasqal_literal*));
+  else
+    rtsc->source_literals = NULL;
 
   rtsc->query = rdf_query;
 
@@ -481,9 +272,9 @@ rasqal_raptor_init_triples_source(rasqal_query* rdf_query,
     }
 
     rtsc->mapped_id_base = rasqal_query_get_genid(rdf_query,
-                                                  (const unsigned char*)"graphid",
+                                                  RASQAL_GOOD_CAST(const unsigned char*, "graphid"),
                                                   i);
-    rtsc->mapped_id_base_len = strlen((const char*)rtsc->mapped_id_base);
+    rtsc->mapped_id_base_len = strlen(RASQAL_GOOD_CAST(const char*, rtsc->mapped_id_base));
 
     parser_name = dg->format_name;
     if(parser_name) {
@@ -497,19 +288,11 @@ rasqal_raptor_init_triples_source(rasqal_query* rdf_query,
     if(!parser_name)
       parser_name = "guess";
     
-#ifdef HAVE_RAPTOR2_API
     parser = raptor_new_parser(rdf_query->world->raptor_world_ptr, parser_name);
     raptor_parser_set_statement_handler(parser, rtsc, rasqal_raptor_statement_handler);
     raptor_world_set_generate_bnodeid_handler(rdf_query->world->raptor_world_ptr,
                                               rtsc,
                                               rasqal_raptor_generate_id_handler);
-#else
-    parser = raptor_new_parser(parser_name);
-    raptor_set_statement_handler(parser, rtsc, rasqal_raptor_statement_handler);
-    raptor_set_error_handler(parser, rdf_query, rasqal_raptor_error_handler);
-    raptor_set_generate_id_handler(parser, rtsc,
-                                   rasqal_raptor_generate_id_handler);
-#endif
 
 #ifdef RAPTOR_FEATURE_NO_NET
     if(rdf_query->features[RASQAL_FEATURE_NO_NET])
@@ -518,17 +301,9 @@ rasqal_raptor_init_triples_source(rasqal_query* rdf_query,
 #endif
 
     if(iostr) {
-#ifdef HAVE_RAPTOR2_API
       raptor_parser_parse_iostream(parser, iostr, dg->base_uri);
-#else
-      rasqal_raptor_parse_iostream(parser, iostr, dg->base_uri);
-#endif
     } else {
-#ifdef HAVE_RAPTOR2_API
       raptor_parser_parse_uri(parser, uri, name_uri);
-#else
-      raptor_parse_uri(parser, uri, name_uri);
-#endif
     }
     
     raptor_free_parser(parser);
@@ -540,7 +315,7 @@ rasqal_raptor_init_triples_source(rasqal_query* rdf_query,
 
     /* This is freed in rasqal_raptor_free_triples_source() */
     /* rasqal_free_literal(rtsc->source_literal); */
-    RASQAL_FREE(cstring, rtsc->mapped_id_base);
+    RASQAL_FREE(char*, rtsc->mapped_id_base);
     
     if(rdf_query->failed) {
       rasqal_raptor_free_triples_source(user_data);
@@ -569,7 +344,7 @@ rasqal_raptor_new_triples_source(rasqal_query* rdf_query,
  * @world: rasqal_world object
  * @triple: #rasqal_triple to match against
  * @match: #rasqal_triple with wildcards
- * @parts; parts of the triple to match
+ * @parts: parts of the triple to match - XOR of #rasqal_triple_parts bits
  * .
  * 
  * Match a rasqal_triple against a rasqal_triple with NULL
@@ -577,11 +352,11 @@ rasqal_raptor_new_triples_source(rasqal_query* rdf_query,
  * 
  * Return value: non-0 on match
  **/
-static int
+int
 rasqal_raptor_triple_match(rasqal_world* world,
                            rasqal_triple *triple,
                            rasqal_triple *match,
-                           rasqal_triple_parts parts)
+                           unsigned int parts)
 {
   int rc = 0;
   
@@ -653,7 +428,7 @@ rasqal_raptor_triple_present(rasqal_triples_source *rts, void *user_data,
 {
   rasqal_raptor_triples_source_user_data* rtsc;
   rasqal_raptor_triple *triple;
-  rasqal_triple_parts parts = RASQAL_TRIPLE_SPO;
+  unsigned int parts = RASQAL_TRIPLE_SPO;
   
   rtsc = (rasqal_raptor_triples_source_user_data*)user_data;
 
@@ -691,7 +466,8 @@ rasqal_raptor_free_triples_source(void *user_data)
     if(rtsc->source_literals[i])
       rasqal_free_literal(rtsc->source_literals[i]);
   }
-  RASQAL_FREE(raptor_literal_ptr, rtsc->source_literals);
+  if(rtsc->source_literals)
+    RASQAL_FREE(raptor_literal_ptr, rtsc->source_literals);
 }
 
 
@@ -717,6 +493,8 @@ typedef struct {
 
   /* parts of the triple above to match: always (S,P,O) sometimes C */
   rasqal_triple_parts parts;
+
+  unsigned int bind_parts;
 } rasqal_raptor_triples_match_context;
 
 
@@ -824,8 +602,38 @@ rasqal_raptor_next_match(struct rasqal_triples_match_s* rtm, void *user_data)
 
   rtmc = (rasqal_raptor_triples_match_context*)rtm->user_data;
 
+#if 0
+  if(rtmc->bind_parts & RASQAL_TRIPLE_SUBJECT) {
+    rasqal_variable* v = rasqal_literal_as_variable(rtmc->cur->triple->subject);
+    if(v)
+      rasqal_variable_set_value(v, NULL);
+  }
+  if(rtmc->bind_parts & RASQAL_TRIPLE_PREDICATE) {
+    rasqal_variable* v = rasqal_literal_as_variable(rtmc->cur->triple->predicate);
+    if(v)
+      rasqal_variable_set_value(v, NULL);
+  }
+  if(rtmc->bind_parts & RASQAL_TRIPLE_OBJECT) {
+    rasqal_variable* v = rasqal_literal_as_variable(rtmc->cur->triple->object);
+    if(v)
+      rasqal_variable_set_value(v, NULL);
+  }
+  if(rtmc->bind_parts & RASQAL_TRIPLE_ORIGIN) {
+    rasqal_variable* v = rasqal_literal_as_variable(rtmc->cur->triple->origin);
+    if(v)
+      rasqal_variable_set_value(v, NULL);
+  }
+#endif
+
   while(rtmc->cur) {
     rtmc->cur = rtmc->cur->next;
+#ifdef RASQAL_DEBUG
+    if(!rtmc->cur) {
+      RASQAL_DEBUG1("triple match ended when matching ");
+      rasqal_triple_print(&rtmc->match, stderr);
+      fputc('\n', stderr);
+    }
+#endif
     if(rtmc->cur &&
        rasqal_raptor_triple_match(rtm->world, rtmc->cur->triple, &rtmc->match,
                                   rtmc->parts))
@@ -884,7 +692,7 @@ rasqal_raptor_init_triples_match(rasqal_triples_match* rtm,
   rtm->is_end = rasqal_raptor_is_end;
   rtm->finish = rasqal_raptor_finish_triples_match;
 
-  rtmc = (rasqal_raptor_triples_match_context*)RASQAL_CALLOC(rasqal_raptor_triples_match_context, 1, sizeof(rasqal_raptor_triples_match_context));
+  rtmc = RASQAL_CALLOC(rasqal_raptor_triples_match_context*, 1, sizeof(*rtmc));
   if(!rtmc)
     return -1;
 
@@ -893,12 +701,18 @@ rasqal_raptor_init_triples_match(rasqal_triples_match* rtm,
   rtmc->source_context = rtsc;
   rtmc->cur = rtsc->head;
   
+  /* Parts we bind */
+  rtmc->bind_parts = m->parts;
+
   /* at least one of the triple terms is a variable and we need to
    * do a triplesMatching() over the list of stored raptor_statements
    */
 
   if((var = rasqal_literal_as_variable(t->subject))) {
-    if(var->value)
+    if(rtmc->bind_parts & RASQAL_TRIPLE_SUBJECT)
+      /* we bind it so reset it */
+      rasqal_variable_set_value(var, NULL);
+    else if(var->value)
       rtmc->match.subject = rasqal_new_literal_from_literal(var->value);
   } else
     rtmc->match.subject = rasqal_new_literal_from_literal(t->subject);
@@ -907,7 +721,10 @@ rasqal_raptor_init_triples_match(rasqal_triples_match* rtm,
   
 
   if((var = rasqal_literal_as_variable(t->predicate))) {
-    if(var->value)
+    if(rtmc->bind_parts & RASQAL_TRIPLE_PREDICATE)
+      /* we bind it so reset it */
+      rasqal_variable_set_value(var, NULL);
+    else if(var->value)
       rtmc->match.predicate = rasqal_new_literal_from_literal(var->value);
   } else
     rtmc->match.predicate = rasqal_new_literal_from_literal(t->predicate);
@@ -916,7 +733,10 @@ rasqal_raptor_init_triples_match(rasqal_triples_match* rtm,
   
 
   if((var = rasqal_literal_as_variable(t->object))) {
-    if(var->value)
+    if(rtmc->bind_parts & RASQAL_TRIPLE_OBJECT)
+      /* we bind it so reset it */
+      rasqal_variable_set_value(var, NULL);
+    else if(var->value)
       rtmc->match.object = rasqal_new_literal_from_literal(var->value);
   } else
     rtmc->match.object = rasqal_new_literal_from_literal(t->object);
@@ -927,7 +747,10 @@ rasqal_raptor_init_triples_match(rasqal_triples_match* rtm,
 
   if(t->origin) {
     if((var = rasqal_literal_as_variable(t->origin))) {
-      if(var->value)
+    if(rtmc->bind_parts & RASQAL_TRIPLE_ORIGIN)
+      /* we bind it so reset it */
+      rasqal_variable_set_value(var, NULL);
+    else if(var->value)
         rtmc->match.origin = rasqal_new_literal_from_literal(var->value);
     } else
       rtmc->match.origin = rasqal_new_literal_from_literal(t->origin);
