@@ -86,9 +86,9 @@ static char *program=NULL;
 
 #ifdef RASQAL_INTERNAL
 /* add 'g:' */
-#define GETOPT_STRING "cd:D:e:f:F:g:G:hi:np:r:qs:vw"
+#define GETOPT_STRING "cd:D:e:Ef:F:g:G:hi:np:r:qs:vW:"
 #else
-#define GETOPT_STRING "cd:D:e:f:F:G:hi:np:r:qs:vw"
+#define GETOPT_STRING "cd:D:e:Ef:F:G:hi:np:r:qs:vW:"
 #endif
 
 #ifdef HAVE_GETOPT_LONG
@@ -104,11 +104,9 @@ static struct option long_options[] =
   {"dump-query", 1, 0, 'd'},
   {"data", 1, 0, 'D'},
   {"exec", 1, 0, 'e'},
+  {"ignore-errors", 0, 0, 'E'},
   {"feature", 1, 0, 'f'},
   {"format", 1, 0, 'F'},
-#ifdef RASQAL_INTERNAL
-  {"engine", 1, 0, 'g'},
-#endif
   {"named", 1, 0, 'G'},
   {"help", 0, 0, 'h'},
   {"input", 1, 0, 'i'},
@@ -118,7 +116,7 @@ static struct option long_options[] =
   {"results", 1, 0, 'r'},
   {"source", 1, 0, 's'},
   {"version", 0, 0, 'v'},
-  {"walk-query", 0, 0, 'w'},
+  {"warnings", 1, 0, 'W'},
 #ifdef STORE_RESULTS_FLAG
   {"store-results", 1, 0, STORE_RESULTS_FLAG},
 #endif
@@ -128,6 +126,10 @@ static struct option long_options[] =
 
 
 static int error_count = 0;
+static int warning_count = 0;
+
+static int warning_level = -1;
+static int ignore_errors = 0;
 
 static const char *title_format_string = "Rasqal RDF query utility %s\n";
 
@@ -140,32 +142,44 @@ static const char *title_format_string = "Rasqal RDF query utility %s\n";
 #define MAX_QUERY_ERROR_REPORT_LEN 512
 
 
-#ifdef HAVE_RAPTOR2_API
 static void
-roqet_log_handler(void* user_data, raptor_log_message *message)
+roqet_log_handler(void *data, raptor_log_message *message)
 {
-  /* Only interested in errors and more severe */
-  if(message->level < RAPTOR_LOG_LEVEL_ERROR)
-    return;
+  switch(message->level) {
+    case RAPTOR_LOG_LEVEL_FATAL:
+    case RAPTOR_LOG_LEVEL_ERROR:
+      if(!ignore_errors) {
+        fprintf(stderr, "%s: Error - ", program);
+        raptor_locator_print(message->locator, stderr);
+        fprintf(stderr, " - %s\n", message->text);
+      }
 
-  fprintf(stderr, "%s: Error - ", program);
-  raptor_locator_print(message->locator, stderr);
-  fprintf(stderr, " - %s\n", message->text);
+      error_count++;
+      break;
+
+    case RAPTOR_LOG_LEVEL_WARN:
+      if(warning_level > 0) {
+        fprintf(stderr, "%s: Warning - ", program);
+        raptor_locator_print(message->locator, stderr);
+        fprintf(stderr, " - %s\n", message->text);
+      }
+
+      warning_count++;
+      break;
+
+    case RAPTOR_LOG_LEVEL_NONE:
+    case RAPTOR_LOG_LEVEL_TRACE:
+    case RAPTOR_LOG_LEVEL_DEBUG:
+    case RAPTOR_LOG_LEVEL_INFO:
+
+      fprintf(stderr, "%s: Unexpected %s message - ", program,
+              raptor_log_level_get_label(message->level));
+      raptor_locator_print(message->locator, stderr);
+      fprintf(stderr, " - %s\n", message->text);
+      break;
+  }
 
-  error_count++;
 }
-#else
-static void
-roqet_error_handler(void *user_data, 
-                    raptor_locator* locator, const char *message) 
-{
-  fprintf(stderr, "%s: Error - ", program);
-  raptor_print_locator(stderr, locator);
-  fprintf(stderr, " - %s\n", message);
-
-  error_count++;
-}
-#endif
 
 #define SPACES_LENGTH 80
 static const char spaces[SPACES_LENGTH + 1] = "                                                                                ";
@@ -347,8 +361,23 @@ roqet_query_walk(rasqal_query *rq, FILE *fh, int indent) {
   
   seq = rasqal_query_get_bound_variable_sequence(rq);
   if(seq && raptor_sequence_size(seq) > 0) {
-    fprintf(fh, "query bound variables (%d): ", 
-            raptor_sequence_size(seq));
+    int size = raptor_sequence_size(seq);
+    fprintf(fh, "query projected variable names (%d): ", size);
+    i = 0;
+    while(1) {
+      rasqal_variable* v = (rasqal_variable*)raptor_sequence_get_at(seq, i);
+      if(!v)
+        break;
+
+      if(i > 0)
+        fputs(", ", fh);
+
+      fputs((const char*)v->name, fh);
+      i++;
+    }
+    fputc('\n', fh);
+
+    fprintf(fh, "query bound variables (%d): ", size);
     i = 0;
     while(1) {
       rasqal_variable* v = (rasqal_variable*)raptor_sequence_get_at(seq, i);
@@ -439,6 +468,7 @@ roqet_query_walk(rasqal_query *rq, FILE *fh, int indent) {
 
 typedef enum {
   QUERY_OUTPUT_UNKNOWN,
+  QUERY_OUTPUT_NONE,
   QUERY_OUTPUT_DEBUG,
   QUERY_OUTPUT_STRUCTURE,
   QUERY_OUTPUT_SPARQL,
@@ -447,6 +477,7 @@ typedef enum {
 
 const char* query_output_format_labels[QUERY_OUTPUT_LAST + 1][2] = {
   { NULL, NULL },
+  { "none", "No debug data" },
   { "debug", "Debug query dump (output format may change)" },
   { "structure", "Query structure walk (output format may change)" },
   { "sparql", "SPARQL" }
@@ -536,37 +567,23 @@ print_graph_result(rasqal_query* rq,
   }
   
   /* Declare any query namespaces in the output serializer */
-#ifdef HAVE_RAPTOR2_API
   for(i = 0; (prefix = rasqal_query_get_prefix(rq, i)); i++)
     raptor_serializer_set_namespace(serializer, prefix->uri, prefix->prefix);
   raptor_serializer_start_to_file_handle(serializer, base_uri, output);
-#else
-  for(i = 0; (prefix = rasqal_query_get_prefix(rq, i)); i++)
-    raptor_serialize_set_namespace(serializer, prefix->uri, prefix->prefix);
-  raptor_serialize_start_to_file_handle(serializer, base_uri, output);
-#endif
   
   while(1) {
     raptor_statement *rs = rasqal_query_results_get_triple(results);
     if(!rs)
       break;
 
-#ifdef HAVE_RAPTOR2_API
     raptor_serializer_serialize_statement(serializer, rs);
-#else
-    raptor_serialize_statement(serializer, rs);
-#endif
     triple_count++;
     
     if(rasqal_query_results_next_triple(results))
       break;
   }
   
-#ifdef HAVE_RAPTOR2_API
   raptor_serializer_serialize_end(serializer);
-#else
-  raptor_serialize_end(serializer);
-#endif
   raptor_free_serializer(serializer);
   
   if(!quiet)
@@ -589,9 +606,9 @@ print_formatted_query_results(rasqal_world* world,
   rasqal_query_results_formatter* results_formatter;
   int rc = 0;
   
-  results_formatter = rasqal_new_query_results_formatter2(world,
-                                                          result_format,
-                                                          NULL, NULL);
+  results_formatter = rasqal_new_query_results_formatter(world,
+                                                         result_format,
+                                                         NULL, NULL);
   if(!results_formatter) {
     fprintf(stderr, "%s: Invalid bindings result format `%s'\n",
             program, result_format);
@@ -623,7 +640,7 @@ print_formatted_query_results(rasqal_world* world,
 
 static rasqal_query_results*
 roqet_call_sparql_service(rasqal_world* world, raptor_uri* service_uri,
-                          const char* query_string,
+                          const unsigned char* query_string,
                           raptor_sequence* data_graphs,
                           const char* format)
 {
@@ -651,7 +668,7 @@ roqet_call_sparql_service(rasqal_world* world, raptor_uri* service_uri,
 static rasqal_query*
 roqet_init_query(rasqal_world *world, 
                  const char* ql_name,
-                 const char* ql_uri, const char* query_string,
+                 const char* ql_uri, const unsigned char* query_string,
                  raptor_uri* base_uri,
                  rasqal_feature query_feature, int query_feature_value,
                  const unsigned char* query_feature_string_value,
@@ -659,7 +676,6 @@ roqet_init_query(rasqal_world *world,
                  raptor_sequence* data_graphs)
 {
   rasqal_query* rq;
-  rasqal_query_results* results = NULL;
 
   rq = rasqal_new_query(world, (const char*)ql_name,
                         (const unsigned char*)ql_uri);
@@ -669,11 +685,6 @@ roqet_init_query(rasqal_world *world,
     goto tidy_query;
   }
   
-#ifdef HAVE_RAPTOR2_API
-#else
-  rasqal_query_set_error_handler(rq, world, roqet_error_handler);
-  rasqal_query_set_fatal_error_handler(rq, world, roqet_error_handler);
-#endif
 
   if(query_feature_value >= 0)
     rasqal_query_set_feature(rq, query_feature, query_feature_value);
@@ -686,27 +697,14 @@ roqet_init_query(rasqal_world *world,
     rasqal_query_set_store_results(rq, store_results);
 #endif
   
-  if(data_graphs) {
-    rasqal_data_graph* dg;
-    
-    while((dg = (rasqal_data_graph*)raptor_sequence_pop(data_graphs))) {
-      if(rasqal_query_add_data_graph2(rq, dg)) {
-        fprintf(stderr, "%s: Failed to add data graph to query\n",
-                program);
-        rasqal_free_query_results(results); results = NULL;
-        goto tidy_query;
-      }
-    }
-  }
-
-  if(rasqal_query_prepare(rq, (const unsigned char*)query_string, base_uri)) {
+  if(rasqal_query_prepare(rq, query_string, base_uri)) {
     size_t len = strlen((const char*)query_string);
     
     fprintf(stderr, "%s: Parsing query '", program);
     if(len > MAX_QUERY_ERROR_REPORT_LEN) {
       (void)fwrite(query_string, MAX_QUERY_ERROR_REPORT_LEN, sizeof(char),
                    stderr);
-      fprintf(stderr, "...' (%d bytes) failed\n", (int)len);
+      fprintf(stderr, "...' (%d bytes) failed\n", RASQAL_BAD_CAST(int, len));
     } else {
       (void)fwrite(query_string, len, sizeof(char), stderr);
       fputs("' failed\n", stderr);
@@ -716,6 +714,18 @@ roqet_init_query(rasqal_world *world,
     goto tidy_query;
   }
 
+  if(data_graphs) {
+    rasqal_data_graph* dg;
+    
+    while((dg = (rasqal_data_graph*)raptor_sequence_pop(data_graphs))) {
+      if(rasqal_query_add_data_graph(rq, dg)) {
+        fprintf(stderr, "%s: Failed to add data graph to query\n",
+                program);
+        goto tidy_query;
+      }
+    }
+  }
+
   tidy_query:
   return rq;
 }
@@ -730,6 +740,9 @@ void roqet_print_query(rasqal_query* rq,
   fprintf(stderr, "Query:\n");
   
   switch(output_format) {
+    case QUERY_OUTPUT_NONE:
+      break;
+
     case QUERY_OUTPUT_DEBUG:
       rasqal_query_print(rq, stdout);
       break;
@@ -757,6 +770,95 @@ void roqet_print_query(rasqal_query* rq,
 }
 
 
+static void
+print_help(rasqal_world* world, raptor_world* raptor_world_ptr)
+{
+  int i;
+    
+  printf(title_format_string, rasqal_version_string);
+  puts("Run an RDF query giving variable bindings or RDF triples.");
+  printf("Usage: %s [OPTIONS] <query URI> [base URI]\n", program);
+  printf("       %s [OPTIONS] -e <query string> [base URI]\n", program);
+  printf("       %s [OPTIONS] -p <SPARQL protocol service URI> -e <query string> [base URI]\n\n", program);
+  
+  fputs(rasqal_copyright_string, stdout);
+  fputs("\nLicense: ", stdout);
+  puts(rasqal_license_string);
+  fputs("Rasqal home page: ", stdout);
+  puts(rasqal_home_url_string);
+  
+  puts("\nNormal operation is to execute the query retrieved from URI <query URI>");
+  puts("and print the results in a simple text format.");
+  puts("\nMain options:");
+  puts(HELP_TEXT("e", "exec QUERY      ", "Execute QUERY string instead of <query URI>"));
+  puts(HELP_TEXT("p", "protocol URI    ", "Execute QUERY against a SPARQL protocol service URI"));
+  puts(HELP_TEXT("i", "input LANGUAGE  ", "Set query language name to one of:"));
+  for(i = 0; 1; i++) {
+    const raptor_syntax_description* desc;
+    
+    desc = rasqal_world_get_query_language_description(world, i);
+    if(!desc)
+      break;
+    
+    printf("    %-15s         %s", desc->names[0], desc->label);
+    if(!i)
+      puts(" (default)");
+    else
+      putchar('\n');
+  }
+  
+  puts(HELP_TEXT("r", "results FORMAT  ", "Set query results output format to one of:"));
+  puts("    For variable bindings and boolean results:");
+  puts("      simple                A simple text format (default)");
+  
+  for(i = 0; 1; i++) {
+    const raptor_syntax_description* desc;
+    
+    desc = rasqal_world_get_query_results_format_description(world, i);
+    if(!desc)
+      break;
+    
+    if(desc->flags & RASQAL_QUERY_RESULTS_FORMAT_FLAG_WRITER)
+      printf("      %-10s            %s\n", desc->names[0], desc->label);
+  }
+  
+  puts("    For RDF graph results:");
+  
+  for(i = 0; 1; i++) {
+    const raptor_syntax_description *desc;
+    desc = raptor_world_get_parser_description(raptor_world_ptr, i);
+    if(!desc)
+      break;
+    
+    printf("      %-15s       %s", desc->names[0], desc->label);
+    if(!i)
+      puts(" (default)");
+    else
+      putchar('\n');
+  }
+  puts("\nAdditional options:");
+  puts(HELP_TEXT("c", "count             ", "Count triples - no output"));
+  puts(HELP_TEXT("d FORMAT", "dump-query FORMAT", HELP_PAD "Print the parsed query out in FORMAT:"));
+  for(i = 1; i <= QUERY_OUTPUT_LAST; i++)
+    printf("      %-15s         %s\n", query_output_format_labels[i][0],
+           query_output_format_labels[i][1]);
+  puts(HELP_TEXT("D URI", "data URI      ", "RDF data source URI"));
+  puts(HELP_TEXT("E", "ignore-errors     ", "Ignore error messages"));
+  puts(HELP_TEXT("f FEATURE(=VALUE)", "feature FEATURE(=VALUE)", HELP_PAD "Set query features" HELP_PAD "Use `-f help' for a list of valid features"));
+  puts(HELP_TEXT("F NAME", "format NAME  ", "Set data source format name (default: guess)"));
+  puts(HELP_TEXT("G URI", "named URI     ", "RDF named graph data source URI"));
+  puts(HELP_TEXT("h", "help              ", "Print this help, then exit"));
+  puts(HELP_TEXT("n", "dryrun            ", "Prepare but do not run the query"));
+  puts(HELP_TEXT("q", "quiet             ", "No extra information messages"));
+  puts(HELP_TEXT("s URI", "source URI    ", "Same as `-G URI'"));
+  puts(HELP_TEXT("v", "version           ", "Print the Rasqal version"));
+  puts(HELP_TEXT("W LEVEL", "warnings LEVEL", HELP_PAD "Set warning message LEVEL from 0: none to 100: all"));
+#ifdef STORE_RESULTS_FLAG
+  puts(HELP_TEXT_LONG("store-results BOOL", "DEBUG: Set store results yes/no BOOL"));
+#endif
+  puts("\nReport bugs to http://bugs.librdf.org/");
+}
+
 
 /* Default parser for input graphs */
 #define DEFAULT_DATA_GRAPH_FORMAT "guess"
@@ -767,7 +869,7 @@ int
 main(int argc, char *argv[]) 
 { 
   int query_from_string = 0;
-  void *query_string = NULL;
+  unsigned char *query_string = NULL;
   unsigned char *uri_string = NULL;
   int free_uri_string = 0;
   unsigned char *base_uri_string = NULL;
@@ -787,7 +889,7 @@ main(int argc, char *argv[])
   int dryrun = 0;
   raptor_sequence* data_graphs = NULL;
   const char *result_format = NULL;
-  query_output_format output_format = QUERY_OUTPUT_UNKNOWN;
+  query_output_format output_format = QUERY_OUTPUT_NONE;
   rasqal_feature query_feature = (rasqal_feature)-1;
   int query_feature_value= -1;
   unsigned char* query_feature_string_value = NULL;
@@ -795,7 +897,6 @@ main(int argc, char *argv[])
   raptor_world* raptor_world_ptr = NULL;
 #ifdef RASQAL_INTERNAL
   int store_results = -1;
-  const rasqal_query_execution_factory* engine = NULL;
 #endif
   char* data_graph_parser_name = NULL;
   raptor_iostream* iostr = NULL;
@@ -816,9 +917,7 @@ main(int argc, char *argv[])
   }
   
   raptor_world_ptr = rasqal_world_get_raptor(world);
-#ifdef HAVE_RAPTOR2_API
   rasqal_world_set_log_handler(world, world, roqet_log_handler);
-#endif
   
 #ifdef STORE_RESULTS_FLAG
   /* This is for debugging only */
@@ -881,7 +980,7 @@ main(int argc, char *argv[])
 
       case 'e':
         if(optarg) {
-          query_string = optarg;
+          query_string = (unsigned char*)optarg;
           query_from_string = 1;
         }
         break;
@@ -889,10 +988,10 @@ main(int argc, char *argv[])
       case 'f':
         if(optarg) {
           if(!strcmp(optarg, "help")) {
-            int i;
+            unsigned int i;
             
             fprintf(stderr, "%s: Valid query features are:\n", program);
-            for(i = 0; i < (int)rasqal_get_feature_count(); i++) {
+            for(i = 0; i < rasqal_get_feature_count(); i++) {
               const char *feature_name;
               const char *feature_label;
               if(!rasqal_features_enumerate(world, (rasqal_feature)i,
@@ -909,10 +1008,10 @@ main(int argc, char *argv[])
             rasqal_free_world(world);
             exit(0);
           } else {
-            int i;
+            unsigned int i;
             size_t arg_len = strlen(optarg);
             
-            for(i = 0; i < (int)rasqal_get_feature_count(); i++) {
+            for(i = 0; i < rasqal_get_feature_count(); i++) {
               const char *feature_name;
               size_t len;
               
@@ -990,13 +1089,12 @@ main(int argc, char *argv[])
                   program, optarg);
           fprintf(stderr, "Valid arguments are:\n");
           for(i = 0; 1; i++) {
-            const char *help_name;
-            const char *help_label;
-            if(rasqal_languages_enumerate(world, i, 
-                                          &help_name, &help_label, NULL))
+            const raptor_syntax_description* desc;
+            desc = rasqal_world_get_query_language_description(world, i);
+            if(desc == NULL)
               break;
 
-            fprintf(stderr, "  %-12s for %s\n", help_name, help_label);
+            fprintf(stderr, "  %-18s for %s\n", desc->names[0], desc->label);
           }
           usage = 1;
         }
@@ -1093,14 +1191,9 @@ main(int argc, char *argv[])
           }
           
           if(!data_graphs) {
-#ifdef HAVE_RAPTOR2_API
             data_graphs = raptor_new_sequence((raptor_data_free_handler)rasqal_free_data_graph,
                                               NULL);
 
-#else
-            data_graphs = raptor_new_sequence((raptor_sequence_free_handler*)rasqal_free_data_graph,
-                                              NULL);
-#endif
             if(!data_graphs) {
               fprintf(stderr, "%s: Failed to create data graphs sequence\n",
                       program);
@@ -1112,31 +1205,30 @@ main(int argc, char *argv[])
         }
         break;
 
+      case 'W':
+        if(optarg)
+          warning_level = atoi(optarg);
+        else
+          warning_level = 0;
+        rasqal_world_set_warning_level(world, warning_level);
+        break;
+
+      case 'E':
+        ignore_errors = 1;
+        break;
+
       case 'v':
         fputs(rasqal_version_string, stdout);
         fputc('\n', stdout);
         rasqal_free_world(world);
         exit(0);
 
-      case 'w':
-        fprintf(stderr,
-                "%s: WARNING: `-w' is deprecated.  Please use `" HELP_ARG(d, dump-query) " structure' instead.\n",
-                program);
-        output_format = QUERY_OUTPUT_STRUCTURE;
-        break;
-
 #ifdef STORE_RESULTS_FLAG
       case STORE_RESULTS_FLAG:
         store_results = (!strcmp(optarg, "yes") || !strcmp(optarg, "YES"));
         break;
 #endif
 
-#ifdef RASQAL_INTERNAL
-      case 'g':
-        engine = rasqal_query_get_engine_by_name(optarg);
-        break;
-#endif
-
     }
     
   }
@@ -1174,110 +1266,9 @@ main(int argc, char *argv[])
   }
 
   if(help) {
-    int i, j;
-    
-    printf(title_format_string, rasqal_version_string);
-    puts("Run an RDF query giving variable bindings or RDF triples.");
-    printf("Usage: %s [OPTIONS] <query URI> [base URI]\n", program);
-    printf("       %s [OPTIONS] -e <query string> [base URI]\n", program);
-    printf("       %s [OPTIONS] -p <SPARQL protocol service URI> -e <query string> [base URI]\n\n", program);
-
-    fputs(rasqal_copyright_string, stdout);
-    fputs("\nLicense: ", stdout);
-    puts(rasqal_license_string);
-    fputs("Rasqal home page: ", stdout);
-    puts(rasqal_home_url_string);
-
-    puts("\nNormal operation is to execute the query retrieved from URI <query URI>");
-    puts("and print the results in a simple text format.");
-    puts("\nMain options:");
-    puts(HELP_TEXT("e", "exec QUERY      ", "Execute QUERY string instead of <query URI>"));
-    puts(HELP_TEXT("p", "protocol URI    ", "Execute QUERY against a SPARQL protocol service URI"));
-    puts(HELP_TEXT("i", "input LANGUAGE  ", "Set query language name to one of:"));
-    for(i = 0; 1; i++) {
-      const char *help_name;
-      const char *help_label;
-
-      if(rasqal_languages_enumerate(world, i, &help_name, &help_label, NULL))
-        break;
-
-      printf("    %-15s         %s", help_name, help_label);
-      if(!i)
-        puts(" (default)");
-      else
-        putchar('\n');
-    }
-
-    puts(HELP_TEXT("r", "results FORMAT  ", "Set query results output format to one of:"));
-    puts("    For variable bindings and boolean results:");
-    puts("      simple                A simple text format (default)");
-
-#ifdef HAVE_RAPTOR2_API
-    j = raptor_option_get_count();
-#else
-    j = raptor_get_feature_count();
-#endif
-
-    for(i = 0; i < j; i++) {
-      const char *name;
-      const char *label;
-      int qr_flags = 0;
-
-      if(!rasqal_query_results_formats_enumerate(world, i, &name, &label, 
-                                                 NULL, NULL, &qr_flags) &&
-         (qr_flags & RASQAL_QUERY_RESULTS_FORMAT_FLAG_WRITER))
-        printf("      %-10s            %s\n", name, label);
-    }
-
-    puts("    For RDF graph results:");
-
-    for(i = 0; 1; i++) {
-#ifdef HAVE_RAPTOR2_API
-      const raptor_syntax_description *desc;
-      desc = raptor_world_get_parser_description(raptor_world_ptr, i);
-      if(!desc)
-        break;
-
-      printf("      %-15s       %s", desc->names[0], desc->label);
-#else
-      const char *help_name;
-      const char *help_label;
-      if(raptor_serializers_enumerate(i, &help_name, &help_label, NULL, NULL))
-        break;
-
-      printf("      %-15s       %s", help_name, help_label);
-#endif
-      if(!i)
-        puts(" (default)");
-      else
-        putchar('\n');
-    }
-    puts("\nAdditional options:");
-    puts(HELP_TEXT("c", "count             ", "Count triples - no output"));
-    puts(HELP_TEXT("d", "dump-query FORMAT ", "Print the parsed query out in FORMAT:"));
-    puts(HELP_TEXT("D", "data URI          ", "RDF data source URI"));
-    for(i = 1; i <= QUERY_OUTPUT_LAST; i++)
-      printf("      %-15s         %s\n", query_output_format_labels[i][0],
-             query_output_format_labels[i][1]);
-    puts(HELP_TEXT("f FEATURE(=VALUE)", "feature FEATURE(=VALUE)", HELP_PAD "Set query features" HELP_PAD "Use `-f help' for a list of valid features"));
-    puts(HELP_TEXT("F", "format NAME       ", "Set data source format name (default: guess)"));
-#ifdef RASQAL_INTERNAL
-    puts(HELP_TEXT("g", "engine NAME       ", "INTERNAL: Pick execution engine NAME"));
-#endif
-    puts(HELP_TEXT("G", "named URI         ", "RDF named graph data source URI"));
-    puts(HELP_TEXT("h", "help              ", "Print this help, then exit"));
-    puts(HELP_TEXT("n", "dryrun            ", "Prepare but do not run the query"));
-    puts(HELP_TEXT("q", "quiet             ", "No extra information messages"));
-    puts(HELP_TEXT("s", "source URI        ", "Same as `-G URI'"));
-    puts(HELP_TEXT("v", "version           ", "Print the Rasqal version"));
-    puts(HELP_TEXT("w", "walk-query        ", "Print query.  Same as '-d structure'"));
-#ifdef STORE_RESULTS_FLAG
-    puts(HELP_TEXT_LONG("store-results BOOL", "DEBUG: Set store results yes/no BOOL"));
-#endif
-    puts("\nReport bugs to http://bugs.librdf.org/");
-
+    print_help(world, raptor_world_ptr);
     rasqal_free_world(world);
-    
+
     exit(0);
   }
 
@@ -1342,12 +1333,19 @@ main(int argc, char *argv[])
   } else if(query_string) {
     /* NOP - already got it */
   } else if(!uri_string) {
-    query_string = calloc(FILE_READ_BUF_SIZE, 1);
-    rc = fread(query_string, FILE_READ_BUF_SIZE, 1, stdin);
-    if(ferror(stdin)) {
-      fprintf(stderr, "%s: query string stdin read failed - %s\n",
-              program, strerror(errno));
-      return(1);
+    size_t read_len;
+    
+    query_string = (unsigned char*)calloc(FILE_READ_BUF_SIZE, 1);
+    read_len = fread(query_string, FILE_READ_BUF_SIZE, 1, stdin);
+    if(read_len < FILE_READ_BUF_SIZE) {
+      if(ferror(stdin))
+        fprintf(stderr, "%s: query string stdin read failed - %s\n",
+                program, strerror(errno));
+      else
+        fprintf(stderr, "%s: query string stdin read failed - no error\n",
+                program);
+      rc = 1;
+      goto tidy_setup;
     }
 
     query_from_string = 0;
@@ -1355,9 +1353,17 @@ main(int argc, char *argv[])
     raptor_stringbuffer *sb = raptor_new_stringbuffer();
     size_t len;
     FILE *fh;
+    unsigned char* buffer;
+
+    buffer = (unsigned char*)malloc(FILE_READ_BUF_SIZE);
+    if(!buffer) {
+      rc = 1;
+      goto tidy_setup;
+    }
 
     fh = fopen(filename, "r");
     if(!fh) {
+      free(buffer);
       fprintf(stderr, "%s: file '%s' open failed - %s", 
               program, filename, strerror(errno));
       rc = 1;
@@ -1365,7 +1371,6 @@ main(int argc, char *argv[])
     }
     
     while(!feof(fh)) {
-      unsigned char buffer[FILE_READ_BUF_SIZE];
       size_t read_len;
 
       read_len = fread((char*)buffer, 1, FILE_READ_BUF_SIZE, fh);
@@ -1376,18 +1381,21 @@ main(int argc, char *argv[])
         if(ferror(fh)) {
           fprintf(stderr, "%s: file '%s' read failed - %s\n",
                   program, filename, strerror(errno));
+          free(buffer);
           fclose(fh);
-          return(1);
+          rc = 1;
+          goto tidy_setup;
         }
 
         break;
       }
     }
+    free(buffer);
     fclose(fh);
 
     len = raptor_stringbuffer_length(sb);
-    query_string = malloc(len + 1);
-    raptor_stringbuffer_copy_to_string(sb, (unsigned char*)query_string, len);
+    query_string = (unsigned char*)malloc(len + 1);
+    raptor_stringbuffer_copy_to_string(sb, query_string, len);
     raptor_free_stringbuffer(sb);
     query_from_string = 0;
   } else {
@@ -1395,15 +1403,8 @@ main(int argc, char *argv[])
 
     www = raptor_new_www(raptor_world_ptr);
     if(www) {
-#ifndef HAVE_RAPTOR2_API
-      raptor_www_set_error_handler(www, roqet_error_handler, world);
-#endif
-      raptor_www_fetch_to_string(www, uri, &query_string, NULL, malloc);
-#ifdef HAVE_RAPTOR2_API
+      raptor_www_fetch_to_string(www, uri, (void**)&query_string, NULL, malloc);
       raptor_free_www(www);
-#else
-      raptor_www_free(www);
-#endif
     }
 
     if(!query_string || error_count) {
@@ -1422,7 +1423,7 @@ main(int argc, char *argv[])
       fprintf(stderr, "%s: Calling SPARQL service at URI %s", program,
               service_uri_string);
       if(query_string)
-        fprintf(stderr, " with query '%s'", (char*)query_string);
+        fprintf(stderr, " with query '%s'", query_string);
 
       if(base_uri_string)
         fprintf(stderr, " with base URI %s\n", base_uri_string);
@@ -1431,10 +1432,10 @@ main(int argc, char *argv[])
     } else if(query_from_string) {
       if(base_uri_string)
         fprintf(stderr, "%s: Running query '%s' with base URI %s\n", program,
-                (char*)query_string, base_uri_string);
+                query_string, base_uri_string);
       else
         fprintf(stderr, "%s: Running query '%s'\n", program,
-                (char*)query_string);
+                query_string);
     } else if(filename) {
       if(base_uri_string)
         fprintf(stderr, "%s: Querying from file %s with base URI %s\n", program,
@@ -1473,15 +1474,11 @@ main(int argc, char *argv[])
       goto tidy_query;
     }
 
-    if(output_format != QUERY_OUTPUT_UNKNOWN && !quiet)
+    if(output_format != QUERY_OUTPUT_NONE && !quiet)
       roqet_print_query(rq, raptor_world_ptr, output_format, base_uri);
     
     if(!dryrun) {
-#ifdef RASQAL_INTERNAL
-      results = rasqal_query_execute_with_engine(rq, engine);
-#else
       results = rasqal_query_execute(rq);
-#endif
     }
 
   }
@@ -1528,6 +1525,9 @@ main(int argc, char *argv[])
   if(!query_from_string)
     free(query_string);
 
+  if(rq)
+    rasqal_free_query(rq);
+
  tidy_setup:
 
   if(data_graphs)
@@ -1544,6 +1544,12 @@ main(int argc, char *argv[])
     raptor_free_uri(service_uri);
 
   rasqal_free_world(world);
+
+  if(error_count && !ignore_errors)
+    return 1;
+
+  if(warning_count && warning_level != 0)
+    return 2;
   
   return (rc);
 }
